<?php
if( ! defined( 'ABSPATH' ) ) return;

//https://docs.woothemes.com/document/payment-gateway-api/

if ( class_exists( 'WC_Payment_Gateway' ) ) :

class inicis_card_gateway extends WC_Payment_Gateway {

	/** @var array Array of locales */
	public $locale;
    public $locale_setting = array();
    public $config = array();
    public $checkout;
    public $gnupay_inicis_card;
    public $log_enabled;
	public $gnu_errors = array();

    public function get_the_id() {
        return $this->gnupay_inicis_card;
    }

    public function get_the_icon() {
        return apply_filters('gnupay_inicis_icon', '');
    }

    public function get_the_title() {
        return __( 'INICIS 카드결제', GNUPAY_INICIS );
    }

    public function get_the_description() {
        
        $return_html = '';

        $return_html = __( 'INICIS 카드결제입니다.', GNUPAY_INICIS );
        
        return $return_html;
    }

	/**
	 * Constructor for the gateway.
	 */
	public function __construct() {
        $pay_ids = gnupay_inicis_get_settings('pay_ids');
        $this->gnupay_inicis_card = $pay_ids['card'];

        $this->id                 = $this->get_the_id();
        $this->icon               = $this->get_the_icon();
        $this->has_fields         = false;
        $this->method_title       = $this->get_the_title();
		$this->method_description = $this->get_the_description();
        $this->log_enabled = true;

		// Define user set variables
		$this->title        = $this->get_option( 'title' );
		$this->description  = $this->get_option( 'description' );
		$this->instructions = $this->get_option( 'instructions', $this->description );
        $this->de_pay_complete_status   = $this->get_option( 'de_pay_complete_status' );
        $this->de_cancel_possible_status   = $this->get_option( 'de_cancel_possible_status' );

        $this->supports           = array(
            'refunds',   //환불관련
            'products',
        );

        $keys = array(
                'de_iche_use',	//계좌이체 결제사용
                'de_vbank_use',	//가상계좌 결제사용
                'de_hp_use',	//휴대폰결제사용
                'de_card_use',	//신용카드결제사용
                'de_card_noint_use',	//시용카드 무이자할부사용
                'de_easy_pay_use',	//PG사 간편결제 버튼 사용
                'de_taxsave_use',	//현금 영수증 발급 사용
                'de_inicis_mid',	//INICIS KEY CODE
                'de_inicis_admin_key',	//INICIS KEY PASSWORD
                'de_inicis_web_signkey',	//INICIS WEB SIGIN KEY
                'de_escrow_use',	//에스크로 사용여부
                'de_card_test',		//결제테스트
                'de_tax_flag_use',	//복합과세 결제
                'de_order_after_action',    //주문
                'de_refund_after_status',   //환불 후 주문상태
            );

        foreach($keys as $key){
            $this->config[$key] = $this->get_inicis_option( $key );
        }

        if( $error = $this->pay_bin_check() ){
        }

		//그누페이 폴더 및 키 파일 체크
		GNUPAY_INICIS()->inicis_folder_check();

		// Load the settings.
		$this->init_form_fields();
        $this->init_settings();

		// Actions
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
    	add_action( 'woocommerce_thankyou_'. $this->id, array( $this, 'thankyou_page' ) );

        // 경로 /woocommerce/templates/checkout/form-checkout.php
        add_action( 'woocommerce_before_checkout_form' , array($this, 'inicis_pay_load') );
        add_action( 'woocommerce_checkout_before_order_review' , array($this, 'inicis_checkout_order') );
        // 경로 /woocommerce/templates/checkout/form-pay.php
        add_action( 'woocommerce_pay_order_before_submit' , array($this, 'inicis_pay_load') );
        add_action( 'woocommerce_pay_order_after_submit' , array($this, 'inicis_checkout_order') );

		if ( ! $this->is_valid_for_use() )
			$this->enabled = 'no';
    }

	public function get_inicis_common_fields() {
		return apply_filters( 'woocommerce_settings_api_form_fields_' . $this->gnupay_inicis_card, $this->form_fields );
	}

    public function process_admin_options(){
        $result = parent::process_admin_options();
    
        if( $result ){
            if( $this->id == $this->gnupay_inicis_card ){     //카드일때만 실행;
                $options = apply_filters( 'woocommerce_settings_api_sanitized_fields_' . $this->id, $this->settings );
                
                $pay_ids = gnupay_inicis_get_settings('pay_ids');
                $store_options = array();

                foreach( $pay_ids as $key=>$pid ){
                    $store_options[$key] = get_option( $this->plugin_id . $pid . '_settings' );
                }
                
                if( isset($options['de_iche_use']) && !empty( $store_options['bank'] ) ){   //계좌이체
                    $store_options['bank']['enabled'] = $options['de_iche_use'] ? 'yes' : 'no';
                    update_option( $this->plugin_id . $pay_ids['bank'] . '_settings', $store_options['bank'] );
                }

                if( isset($options['de_vbank_use']) && !empty( $store_options['vbank'] ) ){   //가상계좌
                    $store_options['vbank']['enabled'] = $options['de_vbank_use'] ? 'yes' : 'no';
                    update_option( $this->plugin_id . $pay_ids['vbank'] . '_settings', $store_options['vbank'] );
                }

                if( isset($options['de_hp_use']) && !empty( $store_options['phone'] ) ){   //핸드폰
                    $store_options['phone']['enabled'] = $options['de_hp_use'] ? 'yes' : 'no';
                    update_option( $this->plugin_id . $pay_ids['phone'] . '_settings', $store_options['phone'] );
                }

                if( isset($options['de_easy_pay_use']) && !empty( $store_options['easy'] ) ){   //간편결제
                    $store_options['easy']['enabled'] = $options['de_easy_pay_use'] ? 'yes' : 'no';
                    update_option( $this->plugin_id . $pay_ids['easy'] . '_settings', $store_options['easy'] );
                }

            }
        }

        return $result;
    }

	public function inicis_common_settings() {

		// Load form_field settings.
		$this->locale_setting = get_option( $this->plugin_id . $this->gnupay_inicis_card . '_settings', null );

		if ( ! $this->locale_setting || ! is_array( $this->locale_setting ) ) {

			$this->locale_setting = array();

			// If there are no settings defined, load defaults.
			if ( $form_fields = $this->get_form_fields() ) {

				foreach ( $form_fields as $k => $v ) {
					$this->locale_setting[ $k ] = isset( $v['default'] ) ? $v['default'] : '';
				}
			}
		}

		if ( ! empty( $this->locale_setting ) && is_array( $this->locale_setting ) ) {
			$this->enabled  = isset( $this->locale_setting['enabled'] ) && $this->locale_setting['enabled'] == 'yes' ? 'yes' : 'no';
		}
	}

    public function is_valid_for_use(){

        $is_vaild = true;

        if( ! in_array( get_woocommerce_currency(), array('KRW', 'AUD', 'ARS', 'CAD', 'CLP', 'COP', 'HKD', 'MXN', 'NZD', 'SGD', 'USD') ) ){
            return false;
        }

        $inicis_options = gp_inicis_get_card_options();

        if( !$inicis_options['de_card_test'] && trim($inicis_options['de_inicis_mid']) == '' ){     //실결제일때 사이트키가 없으면
            return false;
        }

        return $is_vaild;
    }
    
	public function get_inicis_option( $key, $empty_value = null ) {

		if ( empty( $this->locale_setting ) ) {
			$this->inicis_common_settings();
		}

		// Get option default if unset.
		if ( ! isset( $this->locale_setting[ $key ] ) ) {
			$form_fields            = $this->get_inicis_common_fields();
			$this->locale_setting[ $key ] = isset( $form_fields[ $key ]['default'] ) ? $form_fields[ $key ]['default'] : '';
		}

		if ( ! is_null( $empty_value ) && empty( $this->locale_setting[ $key ] ) && '' === $this->locale_setting[ $key ] ) {
			$this->locale_setting[ $key ] = $empty_value;
		}

		return $this->locale_setting[ $key ];
	}

	public function can_refund_order( $order ) {
		return $order && $order->get_order_number();
	}

	/**
	 * Logging method.
	 * @param string $message
	 */
	public function log( $message ) {
		if ( $this->log_enabled ) {
			if ( empty( $this->log ) ) {
				$this->log = new WC_Logger();
			}
			$this->log->add( 'inicis', $message );
		}
	}

    public function process_refund( $order_id, $amount = null, $reason = '' ) { //환불관련 ( 부분환불 ) 관리자 페이지에서만 사용가능

        if( !is_admin() ){
            return false;
        }

        if( !$reason ){
            return new WP_Error( 'error', __( '환불요청사유를 입력해 주세요.', GNUPAY_INICIS ) );
        }

        $order = wc_get_order( $order_id );
        
        if( !in_array($order->get_payment_method(), gnupay_inicis_get_settings('pay_ids')) ){     //그누페이 이니시스 결제가 아닌경우 리턴 
            return;
        }

        try {
            
            if ( ! $this->can_refund_order( $order ) ) {
                //$this->log( 'Refund Failed: No transaction ID' );
                //return new WP_Error( 'error', __( 'Refund Failed: No transaction ID', 'woocommerce' ) );
            }

            $config = $this->config;
            $mod_memo = $reason;    //취소사유
            $tax_mny = $amount;  //과세금액
            $free_mny = 0;     //비과세금액
            $payment_method = get_post_meta( $order_id, '_payment_method', true );
            $od_tax_flag    = get_post_meta( $order_id, '_od_tax_flag', true );   //과세 및 비과세 사용시
			$pay_ids = gnupay_inicis_get_settings('pay_ids');

            if( $od_tax_flag ){     //복합과세이면 다시 계산

                $line_item_qtys         = json_decode( sanitize_text_field( stripslashes( $_POST['line_item_qtys'] ) ), true );
                $line_item_totals       = json_decode( sanitize_text_field( stripslashes( $_POST['line_item_totals'] ) ), true );
                $line_item_tax_totals   = json_decode( sanitize_text_field( stripslashes( $_POST['line_item_tax_totals'] ) ), true );

                // Prepare line items which we are refunding
                $line_items = array();
                $item_ids   = array_unique( array_merge( array_keys( $line_item_qtys, $line_item_totals ) ) );

                foreach ( $item_ids as $item_id ) {
                    $line_items[ $item_id ] = array( 'qty' => 0, 'refund_total' => 0, 'refund_tax' => array() );
                }
                foreach ( $line_item_qtys as $item_id => $qty ) {
                    $line_items[ $item_id ]['qty'] = max( $qty, 0 );
                }
                foreach ( $line_item_totals as $item_id => $total ) {
                    $line_items[ $item_id ]['refund_total'] = wc_format_decimal( $total );
                }
                foreach ( $line_item_tax_totals as $item_id => $tax_totals ) {
                    $line_items[ $item_id ]['refund_tax'] = array_map( 'wc_format_decimal', $tax_totals );
                }

                foreach( $line_items as $key=>$v ){
                    if( !isset($v['qty']) || empty($v['qty']) || empty($v['refund_total']) )
                        continue;

                    $refund_tax = 0;

                    foreach($v['refund_tax'] as $rex){
                        if( empty($rex) ) continue;

                        $refund_tax = $refund_tax + (int) $rex;
                    }

                    if( $v['refund_total'] && empty($refund_tax) ){    //세금이 붙지 않으면 비과세
                        $free_mny = $free_mny + (int) $v['refund_total'];       //비과세 금액을 구합니다.
                    }
                }

                $tax_mny = $amount - $free_mny;  //과세금액을 다시 구합니다.

                $get_total_qty_refunded = $order->get_total_qty_refunded();

                if( !$get_total_qty_refunded ){
                    //return new WP_Error( 'error', __( '복합과세로 결제한 주문은 반드시 수량을 체크해야 합니다.', GNUPAY_INICIS ) );
                }
            }

            $currency      = 'WON';

            $od_currency = get_post_meta($order_id, '_od_currency', true);

            if( in_array( $od_currency, array('AUD', 'ARS', 'CAD', 'CLP', 'COP', 'HKD', 'MXN', 'NZD', 'SGD', 'USD') ) ){   //달러
                $currency = 'USD';
            }

            $oldtid        = get_post_meta( $order_id, '_od_tno', true );
            $price         = $amount;
            $confirm_price = $order->get_remaining_refund_amount();  // 부분취소 잔액
            $buyeremail    = $order->get_billing_email();
            $tax           = isset($refund_tax) ? (int)$refund_tax : 0;
            $taxfree       = (int)$free_mny;

            //return new WP_Error('inicis_refund_error', $confirm_price.' '.$tax.' '.$taxfree );

            $is_refund_success = false;     // 환불 성공/실패 플래그 변수 초기화

            // 환불할 금액이 주문금액과 같고, 취소하면 남은잔액이 0원이면
            if( $order->get_total() && (int) $amount === (int) $order->get_total() && (int) $confirm_price === 0 ){

                // 전액 환불 처리합니다.
                $is_refund_success = $this->order_inicis_refund($order->get_id(), null, $reason);

            } else {
                
                include_once(GNUPAY_INICIS_PATH.'inicis/settle_inicis.inc.php');
                //아래는 부분취소 처리
                /***********************
                 * 3. 재승인 정보 설정 *
                 ***********************/
                $inipay->SetField("type",          "repay");                         // 고정 (절대 수정 불가)
                $inipay->SetField("pgid",          "INIphpRPAY");                    // 고정 (절대 수정 불가)
                $inipay->SetField("subpgip",       "203.238.3.10");                  // 고정
                $inipay->SetField("mid",           $config['de_inicis_mid']);       // 상점아이디
                $inipay->SetField("admin",         $config['de_inicis_admin_key']); //비대칭 사용키 키패스워드
                $inipay->SetField("oldtid",        $oldtid);                         // 취소할 거래의 거래아이디
                $inipay->SetField("currency",      $currency);                       // 화폐단위
                $inipay->SetField("price",         $price);                          // 취소금액
                $inipay->SetField("confirm_price", $confirm_price);                  // 승인요청금액
                $inipay->SetField("buyeremail",    $buyeremail);                     // 구매자 이메일 주소
                $inipay->SetField("tax",           $tax);                            // 부가세금액
                $inipay->SetField("taxfree",       $taxfree);                        // 비과세금액


                /******************
                 * 4. 재승인 요청 *
                 ******************/
                $inipay->startAction();


                /*******************************************************************
                 * 5. 재승인 결과                                                  *
                 *                                                                 *
                 * 신거래번호 : $inipay->getResult('TID')                                     *
                 * 결과코드 : $inipay->getResult('ResultCode') ("00"이면 재승인 성공)         *
                 * 결과내용 : $inipay->getResult('ResultMsg') (재승인결과에 대한 설명)        *
                 * 원거래 번호 : $inipay->getResult('PRTC_TID')                                *
                 * 최종결제 금액 : $inipay->getResult('PRTC_Remains')                              *
                 * 부분취소 금액 : $inipay->getResult('PRTC_Price')                          *
                 * 부분취소,재승인 구분값 : $inipay->getResult('PRTC_Type')              *
                 *                          ("0" : 재승인, "1" : 부분취소)         *
                 * 부분취소(재승인) 요청횟수 : $inipay->getResult('PRTC_Cnt')           *
                 *******************************************************************/


                 if($inipay->getResult('ResultCode') == '00') {
                    
                    $is_refund_success = true;

                 } else {
                     return new WP_Error('inicis_refund_error', gp_inicis_iconv_utf8($inipay->GetResult("ResultMsg")).' 코드 : '.$inipay->GetResult("ResultCode") );
                 }

            }   //end if

            if( $is_refund_success ){

                $payment_gateways = gnupay_inicis_get_gateways();

                $current_user = wp_get_current_user();
                $order->add_order_note( sprintf(__( '%s 님이 %s, ( %s ) 이유로 가격 %s 를 취소하셨습니다.', GNUPAY_INICIS ), 
                $current_user->user_login.' ( '.$current_user->ID.' ) ',
                isset( $payment_gateways[ $payment_method ] ) ? esc_html( $payment_gateways[ $payment_method ]->get_title() ) : esc_html( $payment_method ),
                $reason,
                wc_price($amount)
                ) );
                return true;
            } else {
                return new WP_Error('inicis_refund_error', __( '환불에 실패하였습니다.', GNUPAY_INICIS ) );
            }
            
        } catch ( Exception $e ) {

			return new WP_Error( 'inicis_refund_error', $e->getMessage() );

        }

		return false;

    }

    public function order_inicis_refund( $order_id, $amount = null, $reason = '' ) { //환불관련 ( 주문건을 환불처리 합니다. )

        $config = $this->config;
        $inicis_order = wc_get_order( $order_id );

        try {

            $tno = get_post_meta( $order_id, '_od_tno', true );
            if( ! $tno ){
                return new WP_Error('inicis_refund_error', __('pg 거래번호가 저장되지 않았거나 없습니다.', GNUPAY_INICIS));
            }

            include_once(GNUPAY_INICIS_PATH.'inicis/settle_inicis.inc.php');

            /**************************
             * 1. 라이브러리 인클루드 *
             **************************/

            $page_msg = is_admin() ? '관리자주문에서 처리-' : '주문자 본인 취소-';
            $cancel_msg = gp_inicis_iconv_euckr($page_msg.$reason);

            /*********************
             * 3. 취소 정보 설정 *
             *********************/
            $inipay->SetField("type",      "cancel");                        // 고정 (절대 수정 불가)
            $inipay->SetField("mid",       $config['de_inicis_mid']);       // 상점아이디
            /**************************************************************************************************
             * admin 은 키패스워드 변수명입니다. 수정하시면 안됩니다. 1111의 부분만 수정해서 사용하시기 바랍니다.
             * 키패스워드는 상점관리자 페이지(https://iniweb.inicis.com)의 비밀번호가 아닙니다. 주의해 주시기 바랍니다.
             * 키패스워드는 숫자 4자리로만 구성됩니다. 이 값은 키파일 발급시 결정됩니다.
             * 키패스워드 값을 확인하시려면 상점측에 발급된 키파일 안의 readme.txt 파일을 참조해 주십시오.
             **************************************************************************************************/
            $inipay->SetField("admin",     $config['de_inicis_admin_key']); //비대칭 사용키 키패스워드
            $inipay->SetField("tid",       $tno);                   // 취소할 거래의 거래아이디
            $inipay->SetField("cancelmsg", $cancel_msg);        

            /****************
             * 4. 취소 요청 *
             ****************/
            $inipay->startAction();

            /****************************************************************
             * 5. 취소 결과                                           	*
             *                                                        	*
             * 결과코드 : $inipay->getResult('ResultCode') ("00"이면 취소 성공)  	*
             * 결과내용 : $inipay->getResult('ResultMsg') (취소결과에 대한 설명) 	*
             * 취소날짜 : $inipay->getResult('CancelDate') (YYYYMMDD)          	*
             * 취소시각 : $inipay->getResult('CancelTime') (HHMMSS)            	*
             * 현금영수증 취소 승인번호 : $inipay->getResult('CSHR_CancelNum')    *
             * (현금영수증 발급 취소시에만 리턴됨)                          *
             ****************************************************************/

            $res_cd  = $inipay->getResult('ResultCode');
            $res_msg = $inipay->getResult('ResultMsg');

            if($res_cd != '00') {
                return new WP_Error( 'inicis_refund_error', gp_inicis_iconv_utf8($res_msg).' 코드 : '.$res_cd );
            }

            $current_user = wp_get_current_user();

            $cancel_msg = is_admin() ? __('관리자페이지에서 취소처리 되었습니다.', GNUPAY_INICIS ) : sprintf(__( '%s 님의 요청으로 인해 %s 이 환불되었습니다.', GNUPAY_INICIS ), $current_user->user_login.' ( '.$current_user->ID.' ) ', wc_price($amount));

            $inicis_order->add_order_note($cancel_msg);

            return true;

        } catch ( Exception $e ) {

			return new WP_Error( 'inicis_refund_error', $e->getMessage() );

        }

		return false;

    }

    public function inicis_config(){
        $args = wp_parser_args( $this->config, array(

            ));
        return $args;
    }

    public function inicis_pay_load( $checkout ){
        global $wp, $woocommerce;
        
        //체크아웃일때만 작동
        if( !is_checkout() )
            return;

        if( GNUPAY_INICIS()->is_inicis_pay_load ){   //한번만 실행되게 한다.
            return;
        }

        GNUPAY_INICIS()->is_inicis_pay_load = true;

        $config = $this->config;
        $this->checkout = $checkout;

        $goods = '';
        $goods_count = -1;
        $good_info = '';

        $comm_tax_mny = 0; // 과세금액
        $comm_vat_mny = 0; // 부가세
        $comm_free_mny = 0; // 면세금액
        $tot_tax_mny = 0;

        $send_cost  = 0;    //배송비

        $i = 0;

        foreach ( WC()->cart->get_cart() as $cart_item_key => $cart_item ) {
            if( empty($cart_item) ) continue;

            $_product        = wc_get_product( $cart_item['product_id'] );
            $product_title = $_product->get_title();
            $quantity = $cart_item['quantity'];
            $line_total = $cart_item['line_total'];
			$_tax_stats = $_product->get_tax_status();

            if( $quantity > 1 ){
                $product_title .= ' x '.$quantity;
            }

            if (!$goods){
                $goods = preg_replace("/\'|\"|\||\,|\&|\;/", "", $product_title);
            }

            $goods_count++;

            // 에스크로 상품정보
            if($config['de_escrow_use']) {
                if ($i>0)
                    $good_info .= chr(30);
                $good_info .= "seq=".($i+1).chr(31);
                $good_info .= "ordr_numb=#order_replace#_".sprintf("%04d", $i).chr(31);
                $good_info .= "good_name=".addslashes($product_title).chr(31);
                $good_info .= "good_cntx=".$quantity.chr(31);
                $good_info .= "good_amtx=".$line_total.chr(31);
            }

            // 복합과세금액
            if( wc_tax_enabled() && $config['de_tax_flag_use'] == 'yes' ) {
				if($_tax_stats == 'none'){
					$comm_free_mny += $line_total;;
				} else {
					$tot_tax_mny += $line_total;
				}
            }

            $i++;
        }

        if ($goods_count) $goods = sprintf(__('%s 외 %d 건', GNUPAY_INICIS), $goods, $goods_count);

        // 복합과세처리
        if( wc_tax_enabled() && $config['de_tax_flag_use'] == 'yes' ) {
            $comm_tax_mny = round( WC()->cart->cart_contents_total + WC()->cart->shipping_total - $comm_free_mny ); // 과세금액 ( 카트 )
            $comm_vat_mny = round(WC()->cart->get_taxes_total(false, false));    //부가세( 카트 )
        }

        if ($config['de_escrow_use'] == 1) {
            // 에스크로결제 테스트
            $useescrow = ':useescrow';
            if( GNUPAY_INICIS_MOBILE ){    //모바일이면
                $useescrow = '&useescrow=Y';
            }
        }
        else {
            // 일반결제 테스트
            $useescrow = '';
        }

        $info = array(
                'od_id' =>  '',
                'goods' =>  $goods,
                'tot_price' =>  WC()->cart->total,
                'useescrow' => $useescrow,
                'goods_count'   =>  ($goods_count + 1),
                'good_info'     =>  $good_info,
                'comm_tax_mny'      =>  $comm_tax_mny,
                'comm_vat_mny'      =>  $comm_vat_mny,
                'comm_free_mny'  =>  $comm_free_mny,
            );

        include_once(GNUPAY_INICIS_PATH.'inicis/settle_inicis.inc.php');

        GNUPAY_INICIS()->goodsinfo=$info;

        require_once(GNUPAY_INICIS_PATH.'inicis/orderform.1.php');
        
        // 결제대행사별 코드 include (결제대행사 정보 필드)
        require_once(GNUPAY_INICIS_PATH.'inicis/orderform.2.php');

        wp_enqueue_script('jquery');

        if( GNUPAY_INICIS_MOBILE ){ // 모바일이면
            wp_register_script('gnupay_inicis_pay_js', GNUPAY_INICIS_URL.'js/inicis_pay.js', array('jquery'), GNUPAY_INICIS_VERSION , true);
        } else {
            wp_register_script('gnupay_inicis_pay_js', GNUPAY_INICIS_URL.'js/inicis_pay.js', array('jquery', 'gp_inicis_ini_js'), GNUPAY_INICIS_VERSION , true);
        }


        // Localize the script with new data
        $translation_array = array(
            'is_mobile'=>GNUPAY_INICIS_MOBILE,
            'ajaxurl'=>admin_url('admin-ajax.php'),
        );

        if ( ! empty( $wp->query_vars['order-pay'] ) ) {
            $translation_array['order_id'] = $wp->query_vars['order-pay'];
        }

        wp_localize_script( 'gnupay_inicis_pay_js', 'gnupay_inicis_object', $translation_array );

        wp_enqueue_script('gnupay_inicis_pay_js');
    }

    public function inicis_checkout_order(){
        global $wp, $woocommerce;
        
        $config = $this->config;
    }

    public function thankyou_page(){
        echo apply_filters('gnupay_inicis_thankyou_msg', __('결제해 주셔서 감사합니다.', GNUPAY_INICIS), $this->id );
    }

    public function init_form_fields(){
        $this->form_fields = array(
            'enabled' => array(
                'title' => __( 'Enable/Disable', 'woocommerce' ),
                'type' => 'checkbox',
                'label' => __( 'INICIS 카드결제를 활성화합니다.', GNUPAY_INICIS ),
                'default' => ''
            ),
            'title' => array(
                'title' => __( 'Title', 'woocommerce' ),
                'type' => 'text',
                'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce' ),
                'default' => __( 'INICIS 카드결제', GNUPAY_INICIS ),
                'desc_tip'      => true,
            ),
            'description' => array(
                'title' => __( 'Description', 'woocommerce' ),
                'type' => 'textarea',
                'default' => ''
            ),
            'instructions' => array(
                'title' => __('Instructions', 'woocommerce'),
                'type' => 'textarea',
                'description' => __('Instructions that will be added to the thank you page.', 'woocommerce'),
                'default' => '',
                'desc_tip' => true,
            ),
            'de_card_noint_use' => array(
                'title' => __( '신용카드 무이자할부사용', GNUPAY_INICIS ),
                'type' => 'select',
                'options'	=> array(
                    '0'			=> __( '사용안함', GNUPAY_INICIS ),
                    '1'			=> __( '사용', GNUPAY_INICIS ),
                ),
                'description'   =>  __( '주문시 신용카드 무이자할부를 가능하게 할것인지를 설정합니다.<br>사용으로 설정하시면 PG사 가맹점 관리자 페이지에서 설정하신 무이자할부 설정이 적용됩니다.<br>사용안함으로 설정하시면 PG사 무이자 이벤트 카드를 제외한 모든 카드의 무이자 설정이 적용되지 않습니다.', GNUPAY_INICIS ),
            ),
            'de_taxsave_use'    =>  array(
                'title' => __( '현금영수증 발급사용', GNUPAY_INICIS ),
                'type' => 'select',
                'options'	=> array(
                    '0'			=> __( '사용안함', GNUPAY_INICIS ),
                    '1'			=> __( '사용', GNUPAY_INICIS ),
                ),
                'description'   =>  __( '관리자는 설정에 관계없이 주문 보기에서 발급이 가능합니다.<br>현금영수증 발급 취소는 PG사에서 지원하는 현금영수증 취소 기능을 사용하시기 바랍니다.', GNUPAY_INICIS ),
            ),
            'de_inicis_mid' => array(
                'title' => __('KG이니시스 상점키 파일', GNUPAY_INICIS), 
                'type' => 'inicis_key', 
                'description' => __('KG이니시스로 부터 발급 받으신 zip파일( 또는 압축파일 )을 업로드 합니다.<br>만약, 상점아이디가 SIR로 시작하지 않는다면 계약담당자에게 변경 요청을 해주시기 바랍니다. (Tel. 02-3430-5858)<br>주의) 원본파일명 그대로 올려주셔야 합니다.<br>옳은예) SIRpaytest.zip <br>틀린예) SIRpaytest (1).zip <<< 파일이름을 변경하시면 안됩니다.', GNUPAY_INICIS),
            ),
            'de_inicis_admin_key'     =>  array(
                'title' => __( 'KG이니시스 키패스워드', GNUPAY_INICIS ),
                'type' => 'input',
                'description'   =>  __( 'KG이니시스에서 발급받은 4자리 상점 키패스워드를 입력합니다.<br>KG이니시스 상점관리자 패스워드와 관련이 없습니다.<br>키패스워드 값을 확인하시려면 상점측에 발급된 키파일 안의 readme.txt 파일을 참조해 주십시오', GNUPAY_INICIS ),
                'default' => '1111',
				'custom_attributes' => array(
					'data-placeholder' => __('SITE KEY 를 입력해주세요.', GNUPAY_INICIS )
				)
            ),
            'de_inicis_web_signkey'     =>  array(
                'title' => __( 'KG이니시스 웹 signkey', GNUPAY_INICIS ),
                'type' => 'input',
                'description'   =>  __( '웹표준 결제에 필요한 키를 입력합니다.<br>실결제를 하려면 반드시 입력해야 합니다.', GNUPAY_INICIS ),
				'custom_attributes' => array(
					'data-placeholder' => __('웹표준 결제 signkey를 입력해주세요.', GNUPAY_INICIS )
				)
            ),
            'de_escrow_use'     =>  array(
                'title' => __( '에스크로 사용', GNUPAY_INICIS ),
                'type' => 'select',
                'options'	=> array(
                    '0'			=> __( '일반결제 사용', GNUPAY_INICIS ),
                    '1'			=> __( '에스크로결제 사용', GNUPAY_INICIS ),
                ),
                'description'   =>  __( '에스크로 결제를 사용하시려면, 반드시 결제대행사 상점 관리자 페이지에서 에스크로 서비스를 신청하신 후 사용하셔야 합니다.<br>에스크로 사용시 배송과의 연동은 되지 않으며 에스크로 결제만 지원됩니다.', GNUPAY_INICIS ),
            ),
            'de_card_test'     =>  array(
                'title' => __( '결제테스트', GNUPAY_INICIS ),
                'type' => 'select',
                'options'	=> array(
                    '0'			=> __( '실결제', GNUPAY_INICIS ),
                    '1'			=> __( '테스트 결제', GNUPAY_INICIS ),
                ),
                'default' => '1',
            ),
            'de_tax_flag_use'     =>  array(
                'title' => __( '복합과세 결제', GNUPAY_INICIS ),
                'type' => 'checkbox',
                'label' => __( '복합결제 사용', GNUPAY_INICIS ),
                'default' => 'no',
                'description'   =>  __( '복합과세(과세, 비과세) 결제를 사용하려면 체크하십시오.<br >( 우커머스 -> 설정 -> 세금 옵션 -> 세금 활성화 도 같이 체크되어야 합니다. )<br>복합과세 결제를 사용하기 전 PG사에 별도로 결제 신청을 해주셔야 합니다. 사용시 PG사로 문의하여 주시기 바랍니다.', GNUPAY_INICIS ),
            ),
            'de_refund_after_status'     =>  array(
                'title' => __( '환불 후 주문상태', GNUPAY_INICIS ),
                'description'   =>  __( '환불 후 주문상태를 지정합니다.', GNUPAY_INICIS ),
                'type'              => 'select',
                'class'             => 'wc-enhanced-select',
                'css'               => 'width: 450px;',
                'label' => __( '환불 후 주문상태', GNUPAY_INICIS ),
                'default' => 'wc-cancelled',
                'options' => wc_get_order_statuses(),
				'desc_tip'          => true,
				'custom_attributes' => array(
					'data-placeholder' => __('환불 후 주문상태를 선택해 주세요.', GNUPAY_INICIS )
				)
            ),
            'de_pay_complete_status'     =>  array(
                'title' => __( '사용자 결제 후 주문상태', GNUPAY_INICIS ),
                'description'   =>  __( 'INICIS 결제시 사용자의 주문상태를 지정합니다.', GNUPAY_INICIS ),
                'type'              => 'select',
                'class'             => 'wc-enhanced-select',
                'css'               => 'width: 450px;',
                'label' => __( '사용자 결제 후 주문상태', GNUPAY_INICIS ),
                'default' => 'wc-processing',
                'options' => wc_get_order_statuses(),
				'desc_tip'          => true,
				'custom_attributes' => array(
					'data-placeholder' => __('선택해 주세요.', GNUPAY_INICIS )
				)
            ),
            'de_cancel_possible_status'     =>  array(
                'title' => __( '주문취소 가능한 상태 ( 사용자 )', GNUPAY_INICIS ),
                'description'   =>  __( 'INICIS 결제시 사용자가 주문취소 할 수 있는 상태를 지정합니다.', GNUPAY_INICIS ),
                'type'              => 'multiselect',
                'class'             => 'wc-enhanced-select',
                'css'               => 'width: 450px;',
                'label' => __( '주문취소 가능한 상태 ( 사용자 )', GNUPAY_INICIS ),
                'default' => '',
                'options' => wc_get_order_statuses(),
				'desc_tip'          => true,
				'custom_attributes' => array(
					'data-placeholder' => __('주문취소 가능한 상태를 선택해 주세요.', GNUPAY_INICIS )
				)
            ),
        );
    }

    /**
     * Generate key file HTML.
     *
     * @access public
     * @param mixed $key
     * @param mixed $data
     * @since 1.0.0
     * @return string
     */
    public function generate_inicis_key_html( $key, $data ) {

        $field    = $this->plugin_id . $this->id . '_' . $key;
        $defaults = array(
            'class'             => 'button-secondary',
            'css'               => '',
            'custom_attributes' => array(),
            'desc_tip'          => false,
            'description'       => '',
            'title'             => '',
        );

        $data = wp_parse_args( $data, $defaults );

        $key_value = $this->get_option( $key );
        ob_start();
        ?>
        <tr valign="top">
            <th scope="row" class="titledesc">
                <label for="<?php echo esc_attr( $field ); ?>"><?php echo wp_kses_post( $data['title'] ); ?></label>
                <?php echo $this->get_tooltip_html( $data ); ?>
                <br>
                <br>
                <a href="http://sir.kr/main/service/inicis_pg.php" target="_blank" class="button"><?php _e('KG이니시스 서비스신청하기', GNUPAY_INICIS); ?></a>
            </th>
            <td class="forminp">
                <fieldset>
                    <legend class="screen-reader-text"><span><?php echo wp_kses_post( $data['title'] ); ?></span></legend>
                    
                    <input type="hidden" name="<?php echo esc_attr( $field ); ?>" id="<?php echo esc_attr( $field ); ?>" value="<?php echo esc_attr( $key_value ); ?>" >
                    <input type="file" name="gp_inicis_keyfile" class="<?php echo esc_attr( $data['class'] ); ?>" style="<?php echo esc_attr( $data['css'] ); ?>" />
                    
                    <?php echo $this->get_description_html( $data ); ?>
                </fieldset>

                <?php if( $key_value && file_exists(gp_inicis_upload_path().'/keys/SIR'.$key_value.'.zip') ) { ?>
                <div>
                    <input type="checkbox" id="inicis_key_file_del" name="inicis_key_file_del" value="<?php echo $key_value;?>"> <label for="inicis_key_file_del"><span style="color:blue"><?php echo 'SIR'.$key_value.'.zip'; ?></span> <?php _e('파일 삭제', GNUPAY_INICIS);?></label>
                </div>
                <?php } //end if ?>

            </td>
        </tr>
        <?php
        return ob_get_clean();

    }

    public function validate_inicis_key_field($key) {

        // get the file
        $key_name = 'gp_inicis_keyfile';
        $field    = $this->plugin_id . $this->id . '_' . $key;
        $return_value = $_POST[$field];

        // 삭제에 체크가 되어있다면 파일을 삭제합니다.
        if (isset($_POST['inicis_key_file_del']) && $_POST['inicis_key_file_del']) {
            $keyname = sanitize_text_field($_POST['inicis_key_file_del']);
            
            $keyfile = gp_inicis_upload_path().'/keys/SIR'.$keyname.'.zip';

            @unlink($keyfile);

            $inicis_key_folder = GNUPAY_INICIS_KEY_PATH.'/key/SIR'.$keyname.'/';
            
            gp_inicis_rm_rf($inicis_key_folder);

            if( is_dir($inicis_key_folder) ){
                
                $creds = $this->key_credentials($keyname);

                if ( !WP_Filesystem($creds) ) {
                    $no_direct = true;
                    add_action('admin_footer', 'gp_inicis_request_filesystem_credentials_modal', 1);
                    wp_enqueue_script( 'updates' );
                }
            }

            $return_value = '';
        }

        $file = isset($_FILES[ $key_name ]) ? $_FILES[ $key_name ] : '';

        if( !isset($file['tmp_name']) || empty($file['tmp_name']) ){

            return $return_value;
        }

        // check if the API key is longer than 20 characters. Our imaginary API doesn't create keys that large so something must be wrong. Throw an error which will prevent the user from saving.
        if ( isset( $file ) && !empty( $file ) ) {

            $return_value = $result = $this->key_file_upload($file);

            if ( is_wp_error( $result ) ) {
               $this->errors[] = $result->get_error_message();
               $return_value = '';
            } else {
                $return_value = preg_replace('/^sir/i', '', $result);
            }
            
        }

        return $return_value;
    }

    public function pay_bin_check(){

        if( ! is_admin() ){
            return;
        }

        if( GNUPAY_INICIS()->credentials_check ){      //중복방지
            return;
        }
    
        GNUPAY_INICIS()->credentials_check = true;     //중복방지

        $config = $this->config;
	
        if( $config['de_inicis_mid'] && !$config['de_card_test'] && !in_array($config['de_inicis_mid'], array('INIpayTest', 'iniescrow0')) ){
            if( $this->key_file_check($config['de_inicis_mid']) === false ){
                $this->gnu_errors[] = sprintf(__('%s 에 대한 이니시스 키 파일이 존재하지 않습니다.', GNUPAY_INICIS), $config['de_inicis_mid']);
            }
        }

		if( !ini_get('allow_url_fopen') ) {
			//$this->gnu_errors[] = sprintf(__('allow_url_fopen 을 php.ini 에서 활성화 해주세요.', GNUPAY_INICIS), $config['de_inicis_mid']);
		}

        if( ! defined( 'DOING_AJAX' ) ){
            // loop through each error and display it
            foreach ( $this->gnu_errors as $value ) {
                ?>
                <div class="error">
                    <p><?php echo $value; ?></p>
                </div>
                <?php
            }
        }
    }

    public function key_credentials($keyname){
        $page = isset($_GET['page']) ? sanitize_text_field($_GET['page']) : '';
        $tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : '';
        $section = isset($_GET['section']) ? sanitize_text_field($_GET['section']) : '';
        $add_q = $tab ? "&tab=".$tab : '';
        $add_q .= '&inicis_key='.$keyname;

        GNUPAY_INICIS()->credentials_url = $url = wp_nonce_url( admin_url( 'admin.php?page=' . $page. $add_q ) );

        $creds = false;

        ob_start();
        $creds = request_filesystem_credentials($url);
        ob_end_clean();

        return $creds;
    }

    public function key_file_check($keyname, $filename='mcert'){

        if( isset($_FILES['gp_inicis_keyfile']['tmp_name']) && !empty($_FILES['gp_inicis_keyfile']['tmp_name']) ){
            return true;
        }

        $keyname = 'SIR'.$keyname;

        $key_file = GNUPAY_INICIS_KEY_PATH.'/key/'.$keyname.'/mcert.pem';

        if( file_exists($key_file) ){
            return $key_file;
        } else {

            $page = isset($_GET['page']) ? sanitize_text_field($_GET['page']) : '';
            $tab = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : '';
            $section = isset($_GET['section']) ? sanitize_text_field($_GET['section']) : '';
        }

        if($page){

            $add_q = $tab ? "&tab=".$tab : '';

            if( $section ){
                $add_q .= "&section=".$section;
            }

            GNUPAY_INICIS()->credentials_url = $url = wp_nonce_url( admin_url( 'admin.php?page=' . $page. $add_q ) );

            $creds = false;
            
            ob_start();
            $creds = request_filesystem_credentials($url);
            ob_end_clean();

            if ( !WP_Filesystem($creds) ){
                add_action('admin_footer', 'gp_inicis_request_filesystem_credentials_modal', 1);
                wp_enqueue_script( 'updates' );

                $msg = __('이니시스 키 파일 쓰기 권한이 없으므로 연결 정보를 입력해 주세요.', GNUPAY_INICIS);
                wc_enqueue_js('alert("'.$msg.'");');
                return false;
            }

            $unzipfile = unzip_file( gp_inicis_upload_path().'/keys/'.$keyname.'.zip' , GNUPAY_INICIS_KEY_PATH.'/key/'.$keyname.'/' );

            if( is_wp_error($unzipfile) ){
                return false;
            } else {
                return true;
            }
            return false;

        }

        return true;
    }

	/**
	 * Process the payment and return the result.
	 *
	 * @param int $order_id
	 * @return array
	 */
	public function process_payment( $order_id ) {
        global $woocommerce;
        $order = new WC_Order( $order_id );

        $res = wp_parse_args(array(
			'result'    => 'success',
            'order_id'  =>  $order->get_order_number(),
            'order_key' =>  $order->order_key,
			'redirect'  => $this->get_return_url( $order )
		), gnupay_inicis_process_payment($order, $this->config));

		return $res;

	}

    /**
     * Display errors by overriding the display_errors() method
     * @see display_errors()
     */
    public function display_errors( ) {
		
        // loop through each error and display it
        foreach ( $this->errors as $key => $value ) {
            ?>
            <div class="error">
                <p><?php echo $value; ?></p>
            </div>
            <?php
        }
    }


    //키파일 업로드
    public function key_file_upload($files=array()){
        global $wp_filesystem;

        if( !isset($files['tmp_name']) || empty($files['tmp_name']) ){
            return;
        }

        if( substr($files['name'], 0, 3) != 'SIR' ){
            return new WP_Error( GNUPAY_INICIS.'_upload_fail', __( '상점 키파일은 SIR로 시작되는 파일만 업로드 할수 있습니다.', GNUPAY_INICIS) );
        }

        $key_upload_path = '';
        if( $upload_path = gp_inicis_upload_path() ){

            $key_upload_path = $upload_path.'/keys/';

            if(! is_dir($key_upload_path) ){
                wp_mkdir_p( $key_upload_path );
            }
        } else {
            return new WP_Error( GNUPAY_INICIS.'_upload_pemission', __( '업로드 폴더에 쓰기 권한이 없습니다.', GNUPAY_INICIS) );
        }

        if ( class_exists( 'ZipArchive') ) {
            $z = new ZipArchive();
            $zopen = $z->open( $files['tmp_name'], ZIPARCHIVE::CHECKCONS );

            if ( true !== $zopen )
                return new WP_Error( 'incompatible_archive', __( '호환되지 않는 압축파일 입니다.', GNUPAY_INICIS ), array( 'ziparchive_error' => $zopen ) );

            for ( $i = 0; $i < $z->numFiles; $i++ ) {
                $filename = $z->getNameIndex($i);
                if( !in_array( $filename, apply_filters('gp_inicis_key_files', array('keypass.enc', 'mcert.pem', 'mpriv.pem', 'readme.txt'))) ) {
                    return new WP_Error( GNUPAY_INICIS.'_inicis_keyfile_wrong', __( '키 파일이 잘못되었습니다.', GNUPAY_INICIS ) );
                }
            }
        }

        $upload_file = null;
        if( isset($files['tmp_name']) && !empty($files['tmp_name']) ){
            $upload_file = move_uploaded_file($files['tmp_name'], $key_upload_path.$files['name'] );
        }

        if($upload_file){
            WP_Filesystem();
            
            $filepath = pathinfo( $key_upload_path.$files['name'] );
            $unzipfile = unzip_file( $key_upload_path.$files['name'], GNUPAY_INICIS_KEY_PATH.'/key/'.$filepath['filename'].'/');
            
            if( is_wp_error($unzipfile) ){
                //return new WP_Error( GNUPAY_INICIS.'_keyfile_unzip_error', __( '에러가 발생했습니다.', GNUPAY_INICIS ), array( 'keyfile_unzip_error' => $unzipfile ) );
            } else {
                return $filepath['filename'];
            }

            return $filepath['filename'];
        }

        return '';

    }

}   //end class

endif;  //Class exists inicis_card_gateway end if
?>