<?php
if( ! defined( 'ABSPATH' ) ) exit; // 개별 페이지 접근 불가

if( ! function_exists('gp_check_array_sanitize') ){
    //sanitize_text_field check array
    function gp_check_array_sanitize($msg){

        if( is_array($msg) ){
            return array_map( 'sanitize_text_field', $msg );
        }
        return sanitize_text_field($msg);
    }
}

if( ! function_exists('gp_alert') ){
    // 경고메세지를 경고창으로
    function gp_alert($msg='', $url='', $referer_check=false)
    {
        if (!$msg) $msg = __('올바른 방법으로 이용해 주십시오.', GNUPAY_INICIS);

        if ( !$url && $referer_check ){
            $url = wp_get_referer();
        }

        $html = '<meta charset="utf-8">';
        $html .= '<script type="text/javascript">alert("'.$msg.'");';
        if (!$url){
            $html .= 'history.go(-1);';
        }
        $html .= "</script>";

        do_action( 'gp_alert', $msg, $url );
        $html = apply_filters( 'gp_alert', $html, $url );

        echo $html;

        if ($url){
            gp_goto_url($url, true);
        }
        exit;
    }
}

if( ! function_exists('gp_goto_url') ){
    function gp_goto_url($url, $noscript=false)
    {
        $url = str_replace("&amp;", "&", $url);

        if (!headers_sent() && !$noscript)
            header('Location: '.$url);
        else {
            echo '<script>';
            echo 'location.replace("'.$url.'");';
            echo '</script>';
            echo '<noscript>';
            echo '<meta http-equiv="refresh" content="0;url='.$url.'" />';
            echo '</noscript>';
        }
        exit;
    }
}

if( ! function_exists('gp_check_array_sanitize') ){
    //sanitize_text_field check array
    function gp_check_array_sanitize($msg){

        if( is_array($msg) ){
            return array_map( 'sanitize_text_field', $msg );
        }
        return sanitize_text_field($msg);
    }
}

if( ! function_exists('gnupay_inicis_get_settings') ){
    function gnupay_inicis_get_settings($options='gateways'){

        return GNUPAY_INICIS_SETTING::getInstance()->get_options($options);

    }
}

if( ! function_exists('gp_inicis_get_card_options') ){
    function gp_inicis_get_card_options(){

        $plugin_id= apply_filters('gnupay_inicis_parent_plugin_id', 'woocommerce_');
        $pay_ids = gnupay_inicis_get_settings('pay_ids');
        $gnupay_inicis_card_payname = $pay_ids['card'];
        $payment_options = get_option( $plugin_id . $gnupay_inicis_card_payname . '_settings', null );
        
        return $payment_options;
    }
}

if( ! function_exists('gnupay_inicis_get_config_payment') ){
    function gnupay_inicis_get_config_payment($order_id){

        $plugin_id=apply_filters('gnupay_inicis_parent_plugin_id', 'woocommerce_');

        $payment_method = get_post_meta( $order_id, '_payment_method', true );

        $pay_ids = gnupay_inicis_get_settings('pay_ids');
        $gnupay_inicis_card_payname = $pay_ids['card'];
        $payment_options = get_option( $plugin_id . $gnupay_inicis_card_payname . '_settings', null );

        $method_options = get_option( $plugin_id . $payment_method . '_settings', null );

        
        return wp_parse_args($method_options, $payment_options);
    }
}

if( ! function_exists('gnupay_inicis_process_payment') ){
    function gnupay_inicis_process_payment($order, $config){
        $res = array();
        $crr = array(
                'order_id',
                'order_key',
                'payment_method',
                'payment_method_title',
                'billing_last_name',
                'billing_first_name',
                'billing_company',
                'billing_address_1',
                'billing_address_2',
                'billing_city',
                'billing_state',
                'billing_postcode',
                'billing_country',
                'billing_email',
                'billing_phone',
                'shipping_last_name',
                'shipping_first_name',
                'shipping_company',
                'shipping_address_1',
                'shipping_address_2',
                'shipping_city',
                'shipping_state',
                'shipping_postcode',
                'shipping_country',
                'shipping_email',
                'shipping_phone',
                'timestamp',
                'signature',
                'returnurl',
                'mKey',
            );

        foreach($crr as $v){
            $res[$v] = isset($order->$v) ? $order->$v : '';
        }
        
        $order_id = $res['order_id'] = $order->get_order_number();

        $goods = '';
        $goods_count = -1;
        $good_info = '';

        $comm_tax_mny = 0; // 과세금액
        $comm_vat_mny = 0; // 부가세
        $comm_free_mny = 0; // 면세금액
        $tot_tax_mny = 0;

        $send_cost  = 0;    //배송비

        $i = 0;

        foreach ( $order->get_items() as $item_key => $item ) {
            if( empty($item) ) continue;
            
            $_product        = wc_get_product( $item['product_id'] );
            $goods_count += (int) $item['qty'];
            $quantity = $item['qty'];

			$_tax_stats = $_product->get_tax_status();

            if (!$goods){
                $goods = preg_replace("/\'|\"|\||\,|\&|\;/", "", esc_attr($item['name']));
            }

            if( $quantity > 1 ){
                $goods .= ' x '.$quantity;
            }
            $line_total = $item['line_total'];

            // 에스크로 상품정보
            if(!empty($config['de_escrow_use'])) {
                if ($i>0)
                    $good_info .= chr(30);
                $good_info .= "seq=".($i+1).chr(31);
                $good_info .= "ordr_numb=#order_replace#_".sprintf("%04d", $i).chr(31);
                $good_info .= "good_name=".addslashes($goods).chr(31);
                $good_info .= "good_cntx=".$quantity.chr(31);
                $good_info .= "good_amtx=".$line_total.chr(31);
            }

            // 복합과세금액
            if( wc_tax_enabled() && $config['de_tax_flag_use'] == 'yes' ) {
				if($_tax_stats == 'none'){  //비과세이면
					$comm_free_mny += $line_total;
				} else {
					$tot_tax_mny += $line_total;
				}
            }

            $i++;
        }

        if ($goods_count) $goods = sprintf(__('%s 외 %d 건', GNUPAY_INICIS), $goods, $goods_count);

        $res['de_tax_flag_use'] = 0;    //복합과세 체크 초기화

        // 복합과세처리
        if( wc_tax_enabled() && $config['de_tax_flag_use'] == 'yes' ) {     //복합과세이면
            //이게 맞는지 나중에 다시다시 확인할것
            $comm_tax_mny = round($order->get_subtotal() - $comm_free_mny);
            $comm_vat_mny = $order->get_total_tax();
            $res['de_tax_flag_use'] = 1;
        }

        $res['goods'] = $goods;
        $res['tot_price'] = $order->get_total();
        $res['goods_count'] = $goods_count + 1;
        $res['good_info'] = $good_info;
        $res['comm_tax_mny'] = $comm_tax_mny;
        $res['comm_vat_mny'] = $comm_vat_mny;
        $res['comm_free_mny'] = $comm_free_mny;

        if ($config['de_card_test']) {
            if ($config['de_escrow_use'] == 1) {
                // 에스크로결제 테스트
                $config['de_inicis_mid'] = 'iniescrow0';
            }
            else {
                // 일반결제 테스트
                $config['de_inicis_mid'] = 'INIpayTest';
            }
        } else {
            $config['de_inicis_mid'] = "SIR".$config['de_inicis_mid'];

            if ($config['de_escrow_use'] == 1) {
                // 에스크로결제 테스트
                $useescrow = ':useescrow';
                if( GNUPAY_INICIS_MOBILE ){    //모바일이면
                    $useescrow = '&useescrow=Y';
                }
            }
            else {
                // 일반결제 테스트
                $useescrow = '';
            }
        }

        $res['mid'] = $config['de_inicis_mid'];
        $res['useescrow'] = isset($useescrow) ? $useescrow : '';

        $BANK_CODE = array(
            '03' => '기업은행',
            '04' => '국민은행',
            '05' => '외환은행',
            '07' => '수협중앙회',
            '11' => '농협중앙회',
            '20' => '우리은행',
            '23' => 'SC 제일은행',
            '31' => '대구은행',
            '32' => '부산은행',
            '34' => '광주은행',
            '37' => '전북은행',
            '39' => '경남은행',
            '53' => '한국씨티은행',
            '71' => '우체국',
            '81' => '하나은행',
            '88' => '신한은행',
            'D1' => '동양종합금융증권',
            'D2' => '현대증권',
            'D3' => '미래에셋증권',
            'D4' => '한국투자증권',
            'D5' => '우리투자증권',
            'D6' => '하이투자증권',
            'D7' => 'HMC 투자증권',
            'D8' => 'SK 증권',
            'D9' => '대신증권',
            'DA' => '하나대투증권',
            'DB' => '굿모닝신한증권',
            'DC' => '동부증권',
            'DD' => '유진투자증권',
            'DE' => '메리츠증권',
            'DF' => '신영증권'
        );

        $CARD_CODE = array(
            '01' => '외환',
            '03' => '롯데',
            '04' => '현대',
            '06' => '국민',
            '11' => 'BC',
            '12' => '삼성',
            '14' => '신한',
            '15' => '한미',
            '16' => 'NH',
            '17' => '하나 SK',
            '21' => '해외비자',
            '22' => '해외마스터',
            '23' => 'JCB',
            '24' => '해외아멕스',
            '25' => '해외다이너스'
        );

        $PAY_METHOD = array(
            'VCard'      => '신용카드',
            'Card'       => '신용카드',
            'DirectBank' => '계좌이체',
            'HPP'        => '휴대폰',
            'VBank'      => '가상계좌'
        );

        
        $p_mname = apply_filters('gp_inicis_get_p_mname', get_bloginfo('name')); //상점이름
        $mid = $config['de_inicis_mid'];

        if( GNUPAY_INICIS_MOBILE ){    //모바일이면
            $PAY_METHOD = array(
                'ISP'    => '신용카드',
                'CARD'   => '신용카드',
                'BANK'   => '계좌이체',
                'MOBILE' => '휴대폰',
                'VBANK'  => '가상계좌'
            );

            $res['oid'] = apply_filters('gnupay_inicis_get_ordernumber', $mid . "_" . $order_id, $mid, $order_id); // 가맹점 주문번호(가맹점에서 직접 설정)
            $res['p_mname'] = $p_mname;     //상점 이름
            $res['noti_url']   = add_query_arg(array('wc-api'=>'gnupay_inicis_returnurl', 'inicispay'=>'noti'), home_url( '/' ));
            $res['next_url']   = add_query_arg(array('wc-api'=>'gnupay_inicis_returnurl', 'inicispay'=>'next'), home_url( '/' ));
            $res['return_url'] = add_query_arg(array('wc-api'=>'gnupay_inicis_returnurl', 'inicispay'=>'return'), home_url( '/' ));
        } else {
            
            require_once(GNUPAY_INICIS_PATH.'lib/INIStdPayUtil.php');
            $SignatureUtil = new INIStdPayUtil();

            /*
              //*** 위변조 방지체크를 signature 생성 ***

              oid, price, timestamp 3개의 키와 값을

              key=value 형식으로 하여 '&'로 연결한 하여 SHA-256 Hash로 생성 된값

              ex) oid=INIpayTest_1432813606995&price=819000&timestamp=2012-02-01 09:19:04.004


             * key기준 알파벳 정렬

             * timestamp는 반드시 signature생성에 사용한 timestamp 값을 timestamp input에 그대로 사용하여야함
             */

            //############################################
            // 1.전문 필드 값 설정(***가맹점 개발수정***)
            //############################################
            // 여기에 설정된 값은 Form 필드에 동일한 값으로 설정

            $signKey = $config['de_inicis_web_signkey']; // 가맹점에 제공된 웹 표준 사인키(가맹점 수정후 고정);
            
            if( $config['de_card_test'] ){  //테스트결제이면
                $signKey = "SU5JTElURV9UUklQTEVERVNfS0VZU1RS"; // 가맹점에 제공된 웹 표준 사인키(가맹점 수정후 고정);
            }

            $orderNumber = $mid. "_" . $SignatureUtil->getTimestamp(); // 가맹점 주문번호(가맹점에서 직접 설정)
            $timestamp = $SignatureUtil->getTimestamp();   // util에 의해서 자동생성
            $price = $res['tot_price'];
            
            $INIStdPayReturn = add_query_arg(array('wc-api'=>'gnupay_inicis_returnurl', 'inicispay'=>'return'), home_url( '/' ));

            $orderNumber = apply_filters('gnupay_inicis_get_ordernumber', $mid . "_" . $order_id, $mid, $order_id); // 가맹점 주문번호(가맹점에서 직접 설정)

            $cardNoInterestQuota = "11-2:3:,34-5:12,14-6:12:24,12-12:36,06-9:12,01-3:4";  // 카드 무이자 여부 설정(가맹점에서 직접 설정)
            $cardQuotaBase = "2:3:4:5:6:11:12:24:36";  // 가맹점에서 사용할 할부 개월수 설정
            //###################################
            // 2. 가맹점 확인을 위한 signKey를 해시값으로 변경 (SHA-256방식 사용)
            //###################################
            $mKey = $SignatureUtil->makeHash($signKey, "sha256");

            $params = array(
                "oid" => $orderNumber,
                "price" => $price,
                "timestamp" => $timestamp
            );
            
            $sign = $SignatureUtil->makeSignature($params, "sha256");

            $inicis_create = array(
                'signkey' => $signKey,
                'timestamp' => $timestamp,
                'mid' =>   $mid,
                'oid' =>    $orderNumber,
                'cardNoInterestQuota'   =>  $cardNoInterestQuota,
                'cardQuotaBase' =>  $cardQuotaBase,
                'mKey'  => $mKey,
                'signature'  => $sign,
                'returnurl' =>  $INIStdPayReturn,
            );
            
 
            $payment_method = get_post_meta( $order_id, '_payment_method', true );
            $pay_ids = gnupay_inicis_get_settings('pay_ids');

            if( $payment_method == $pay_ids['vbank'] ){ //가상계좌이면

                $plugin_id= apply_filters('gnupay_inicis_parent_plugin_id', 'woocommerce_');
                $pay_options = get_option( $plugin_id . $payment_method . '_settings' );

                if( isset($pay_options['de_deposit_period']) && ! empty($pay_options['de_deposit_period']) ){  //가상계좌 입금 기간 설정

                    $date_ymd = date('Ymd', strtotime("+".absint($pay_options['de_deposit_period'])." days"));
                } else {
                    $date_ymd = date('Ymd', strtotime("+5 days"));
                }

                $inicis_create['acceptmethod'] = 'HPP(2):Card(0):no_receipt:cardpoint:va_receipt:vbanknoreg(0):vbank('.$date_ymd.')';

            }

            $res = apply_filters('gnupay_inicis_create_res', wp_parse_args($inicis_create, $res));
        }

        return $res;
    }
}

function sir_inicis_get_order_status($status){
    $status = preg_replace("/^wc-/i", "", trim($status));

    return $status;
}

function gnupay_inicis_order_test($mid, $config){

    if( in_array( strtolower($mid), array('inipaytest', 'iniescrow0')) ){   //테스트가 맞다면
        return true;
    }

    return false;
}

function gnupay_inicis_get_vbankurl($pay_ids=array()){

    if( !$pay_ids ){
        $pay_ids = gnupay_inicis_get_settings('pay_ids');
    }
    
    return str_replace( 'https:', 'http:', add_query_arg( 'wc-api', $pay_ids['vbank'], home_url( '/' ) ) );

}

if( !function_exists('gp_inicis_order_can_view') ){
    function gp_inicis_order_can_view($order_id){

        $is_can_view = false;

        // Check user has permission to edit
        if ( current_user_can( 'view_order', $order_id ) ) {
            $is_can_view = true;
        }

        if( $retrive_data = WC()->session->get( 'gp_inicis_'.$order_id ) ){
            $is_can_view = true;
        }

        return $is_can_view;
    }
}

function gp_inicis_upload_path($type='dir'){
    $upload_dir = array('error'=>'error');
    try{
        $upload_dir = wp_upload_dir();
    } catch (Exception $e) {
    }
    $path = '';
    if( empty($upload_dir['error']) ){
        $path = apply_filters('sir_get_upload_'.$type, $upload_dir['base'.$type].'/inicis', $upload_dir);
    }
    return $path;
}

// 세션변수 생성
function gp_inicis_set_session($session_name, $value)
{
    if (PHP_VERSION < '5.3.0')
        session_register($session_name);
    // PHP 버전별 차이를 없애기 위한 방법
    $$session_name = $_SESSION[$session_name] = $value;
}

// CHARSET 변경 : euc-kr -> utf-8
function gp_inicis_iconv_utf8($str)
{
    return @iconv('euc-kr', 'utf-8', $str);
}

// CHARSET 변경 : utf-8 -> euc-kr
function gp_inicis_iconv_euckr($str)
{
    return @iconv('utf-8', 'euc-kr', $str);
}

function gp_inicis_replace_oid($oid){

    if( strpos($oid, '_') !== false ){
        $get_order_num = explode('_', $oid);
        $oid = end($get_order_num);
    } else {
        $oid = preg_replace('/\D/', '', $oid);
    }

    return $oid;
}

function gp_inicis_loading_msg($return_url){
    inicis_new_html_header();
    ?>
        <div id="show_progress">
            <span style="display:block; text-align:center;margin-top:120px"><img src="<?php echo GNUPAY_INICIS_URL; ?>/img/loading.gif" alt=""></span>
            <span style="display:block; text-align:center;margin-top:10px; font-size:14px"><?php _e('주문완료 중입니다. 잠시만 기다려 주십시오.', GNUPAY_INICIS); ?></span>
        </div>
        <script type="text/javascript">
            setTimeout( function() {
                location.replace('<?php echo $return_url; ?>');
            }, 300);
        </script>
    <?php
    inicis_new_html_footer();
}


if( !function_exists('gp_inicis_request_filesystem_credentials_modal') ){
    function gp_inicis_request_filesystem_credentials_modal(){
        if( !is_admin() ){
            return;
        }

        if( $url = GNUPAY_INICIS()->credentials_url ){
            $filesystem_method = get_filesystem_method();
            ob_start();
            $filesystem_credentials_are_stored = request_filesystem_credentials($url);
            ob_end_clean();
            $request_filesystem_credentials = ( $filesystem_method != 'direct' && ! $filesystem_credentials_are_stored );
            if ( ! $request_filesystem_credentials ) {
                return;
            }
        ?>
        <div id="request-filesystem-credentials-dialog" class="notification-dialog-wrap request-filesystem-credentials-dialog gp-credentials-dialog">
            <div class="notification-dialog-background"></div>
            <div class="notification-dialog" role="dialog" aria-labelledby="request-filesystem-credentials-title" tabindex="0">
                <div class="request-filesystem-credentials-dialog-content">
                    <?php request_filesystem_credentials( $url ); ?>
                <div>
            </div>
        </div>
        <script>
        jQuery(document).ready(function($) {
            wp.updates.requestForCredentialsModalOpen();

            $(document).on("click", '.gp-credentials-dialog [data-js-action="close"]', function(e){
                e.preventDefault();
                wp.updates.requestForCredentialsModalClose();
            });

            $(document).on('submit', '.gp-credentials-dialog form', function(e){
                e.currentTarget.submit();
            });
        });
        </script>
        <?php
        }   //end if
    }
}

// rm -rf 옵션 : exec(), system() 함수를 사용할 수 없는 서버 또는 win32용 대체
// www.php.net 참고 : pal at degerstrom dot com
function gp_inicis_rm_rf($file)
{
    if (file_exists($file)) {
        if (is_dir($file)) {
            $handle = opendir($file);
            while($filename = readdir($handle)) {
                if ($filename != '.' && $filename != '..')
                    gp_inicis_rm_rf($file.'/'.$filename);
            }
            closedir($handle);

            @chmod($file, 0755);
            @rmdir($file);
        } else {
            @chmod($file, 0644);
            @unlink($file);
        }
    }
}

if( ! function_exists('gnupay_inicis_payment_check') ){
    function gnupay_inicis_payment_check( $order_id, $real_pay, $payment_method, $payment_title, $pay_ids ){

        $real_pay = strtolower( $real_pay );
        $real_payment = '';

        switch( $real_pay ){
            case 'isp' :
            case 'card' :     //신용카드
                $real_payment = $pay_ids['card'];
                break;
            case 'directbank' :     //계좌이체
            case 'bank' :
                $real_payment = $pay_ids['bank'];
                break;
            case 'hpp' :     //휴대폰
            case 'mobile' :
                $real_payment = $pay_ids['phone'];
                break;
            case 'vbank' :     //가상계좌
                $real_payment = $pay_ids['vbank'];
                break;
            case 'kpay' :   // 간편결제
            case 'easypay' :
                $real_payment = $pay_ids['easy'];
                break;
        }

        if( $real_payment && $real_payment != $payment_method ){    //이전것과 비교해서 틀리면

            $available_gateways = WC()->payment_gateways->get_available_payment_gateways();
            $payment_method = $real_payment;

            // Update meta
            update_post_meta( $order_id, '_payment_method', $payment_method );

            $before_pay_title = $payment_title;

            if ( isset( $available_gateways[ $payment_method ] ) ) {
                $payment_title = $available_gateways[ $payment_method ]->get_title();
            } else {
                $payment_title = '';
            }

            update_post_meta( $order_id, '_payment_method_title', $payment_title );

            $current_user = wp_get_current_user();
            $user_name = $current_user->ID ? $current_user->user_login.' ( '.$current_user->ID.' ) ' : __('비회원', GNUPAY_INICIS );

            $order->add_order_note( sprintf(__( '결제 방법이 %s 에서 %s 으로 변경되었습니다. : 사용자 %s ', GNUPAY_INICIS ), 
                $payment_title ? esc_html($payment_title) : esc_html( $payment_method ),
                $before_pay_title,
                $user_name
            ) );

            return array(
                'change'=>1,
                'payment_method'=>$payment_method,
                'payment_title'=>$payment_title
                );
        }

        return false;
    }
}
function gnupay_inicis_get_gateways(){

    $payment_gateways = array();

    if ( WC()->payment_gateways() ) {
        $payment_gateways = WC()->payment_gateways->get_available_payment_gateways();
    }

    return $payment_gateways;
}

if( ! function_exists('inicis_new_html_header') ){
    function inicis_new_html_header($page_mode='', $title=''){
        if( !$title ){
            $title = get_bloginfo('name');
        }
    ?>
    <!DOCTYPE html>
    <!--[if IE 8]>
        <html xmlns="http://www.w3.org/1999/xhtml" class="ie8" <?php language_attributes(); ?>>
    <![endif]-->
    <!--[if !(IE 8) ]><!-->
        <html xmlns="http://www.w3.org/1999/xhtml" <?php language_attributes(); ?>>
    <!--<![endif]-->
    <head>
    <meta http-equiv="X-UA-Compatible" content="IE=Edge">
    <meta http-equiv="Content-Type" content="<?php bloginfo('html_type'); ?>; charset=<?php bloginfo('charset'); ?>" />
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title><?php $title; ?></title>
    <?php do_action( 'inicis_head_new_'.$page_mode ); ?>
    <link rel='stylesheet' id='gnupay-inicis-new'  href='<?php echo GNUPAY_INICIS_URL.'template/new.css'; ?>' type='text/css' media='all' />
    </head>
    <div class="inicis_new_shortcode">
    <?php
    }
}

if( ! function_exists('inicis_new_html_footer') ){
    function inicis_new_html_footer($page_mode=''){
    ?>
    </div>
    <?php do_action( 'inicis_footer_new_'.$page_mode ); ?>
    </body>
    </html>
    <?php
    }
}
?>